/*
bhdev.h
Copyright (C)2001  Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.

*/

#ifndef _BH_POSTDEV_
#define _BH_POSTDEV_

#include "config.h"
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <termios.h>
#include <unistd.h>

#include <gtk--/main.h>

#include <libxml/tree.h>
#include <libxml/parser.h>

#include <usb.h>

#include "postdev.h"

namespace BhPos{
   
/**
 * @defgroup bhdev Devices 
 * @{
 *
 * @ingroup commlib
 * @{
 */


enum BhDevNotifyType{
	BHDEV_NOTIFY_TIMEOUT   = 1,
	BHDEV_NOTIFY_READ      = 2,
	BHDEV_NOTIFY_WRITE     = 4,
	BHDEV_NOTIFY_EXCEPTION = 8
};

/* }@
 *
 * @ingroup bhdev
 * @{
 */

enum BhDevNotify{
   BHDEV_TIMEOUT,
	BHDEV_READ,
	BHDEV_WRITE,
	BHDEV_EXCEPTION,
	BHDEV_ERROR
};

/* }@
 *
 * @ingroup bhdev
 * @{
 */

enum BhDevParam{
	BHDEV_PARAM_TYPE,
   BHDEV_PARAM_DEVICE,
	BHDEV_PARAM_ENABLE,
	BHDEV_PARAM_TOENABLE,
	BHDEV_PARAM_TOVALUE,
	BHDEV_PARAM_TORETRIES,
	BHDEV_PARAM_TOBACKOFF,
	BHDEV_PARAM_TOBOVALUE,
	BHDEV_PARAM_INPUTBUFFSIZE,
	BHDEV_PARAM_OUTPUTBUFFSIZE,
	BHDEV_PARAM_SERIALSPEED,
	BHDEV_PARAM_SERIALBITS,
	BHDEV_PARAM_SERIALSTOPBITS,
	BHDEV_PARAM_SERIALPARITY,
	BHDEV_PARAM_SERIALSOFTFLOW,
	BHDEV_PARAM_SERIALBITSET,
	BHDEV_PARAM_SERIALBITCLR,
	BHDEV_PARAM_SERIALBITGET
};

/* }@
 *
 * @ingroup bhdev
 * @{
 */

enum BhDevSerialParity{
   BHDEV_SERIAL_PARITY_NONE,
   BHDEV_SERIAL_PARITY_ODD,
   BHDEV_SERIAL_PARITY_EVEN
};

/* }@
 *
 * @ingroup bhdev
 * @{
 */

enum BhDevSerialControl{
   BHDEV_SERIAL_CTRL_DTR,
   BHDEV_SERIAL_CTRL_RTS,
   BHDEV_SERIAL_CTRL_CTS,
   BHDEV_SERIAL_CTRL_DSR,
   BHDEV_SERIAL_CTRL_CD,
   BHDEV_SERIAL_CTRL_RI,
   BHDEV_SERIAL_CTRL_RX,
   BHDEV_SERIAL_CTRL_TX
};

/* }@
 *
 * @ingroup bhdev
 * @{
 */

enum BhDevError{
	BHDEV_ERR_NONE=0,
	BHDEV_ERR_DEVNULLPARAM,
	BHDEV_ERR_DEVNAMEALLOC,
	BHDEV_ERR_DEVOPEN,
	BHDEV_ERR_TIMEOUT=100,
	BHDEV_ERR_READ,
	BHDEV_ERR_WRITE
};

/* }@
 *
 * @ingroup bhdev
 * @{
 */

enum BhDevType{
   BHDEV_TYPE_NONE=0,
	BHDEV_TYPE_DESCRIPTOR,
	BHDEV_TYPE_SERIAL,
	BHDEV_TYPE_PARALLEL,
	BHDEV_TYPE_USB,
	BHDEV_TYPE_NETWORK,
	BHDEV_TYPE_FIREWIRE
};

#ifdef _USE_USB
#define READ_FROM_USB               0x45
#define WRITE_TO_USB                0x46
#define START_USB_COMM              0x47
#define END_USB_COMM                0x48
#endif

/* }@
 *
 * @ingroup bhdev
 * @{
 */

class CBhDevice
{
private:
	char     *devname;
	BhDevType type;
	
	int	 fd;
	int	 fd_tag;
	bool	 bEnabled;
#ifdef _USE_USB
   usb_dev_handle  *usb_dev;
#endif	
	bool	 bTimeout;
	int	 timeout;
	int	 timer_tag;
	bool	 bRetry;
	int	 retries;
	int	 max_retries;
	bool	 bBackoff;
	double backoff_value;

protected:	
	BhDevError      bherror;
	int 		       error;
	BhDevNotifyType notifytypes;
	
	int            handle;
	BHDEV_CALLBACK cbFunction;
	
	struct termios old_tio;
	
	char	*indata;
	int	 inlen;
	char	*outdata;
	int	 outlen;
	int	 inread;
	
	int popen();
   
#ifdef _USE_USB
   void usbClose();
   int  usbFind(int type);
   int  usbWrite();
   int  usbRead();
#endif
   
public:
	CBhDevice(BhDevType, int, BHDEV_CALLBACK);
	virtual ~CBhDevice();
	virtual int  Open(const char *, BhDevNotifyType);
	virtual void Close();
	virtual int  Read(int, int, void *);
	virtual int  Write(int, int, void *);
	virtual void Dispatch(unsigned int);
	virtual void SetParameter(BhDevParam, void *);
	virtual void GetParameter(BhDevParam, void *);
	virtual gint Timeout();
	virtual void Active(int d, GdkInputCondition c);
	
	int   GetError();
	char *GetErrorString(int err);
};

/* }@
 *
 * @ingroup bhdev
 * @{
 */

namespace Parser{

class CDeviceParser
{
protected:
	char		 *name;
	char		 *manufacturer;
	char		 *model;
	char 		*filename;
	char 		*protocol;
	xmlNsPtr	ns;
	
	int		  error;
	char      *device;
	int	     type;
	
public:
	CDeviceParser();
	~CDeviceParser();
	int   ParseDevice(xmlDocPtr, xmlNsPtr, xmlNodePtr, CBhDevice&);
	int   SaveDevice(xmlDocPtr, xmlNsPtr, xmlNodePtr, CBhDevice&);
	void  ParseOptions(xmlNodePtr, CBhDevice&);
	int   GetError();
	char *GetErrorString(int);
};

/* }@ */

}

/* }@ */

}

#endif // _BH_POSTDEV_
