/*
bhservice.h
Copyright (C) 2002  Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2004 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 
*/
#ifndef _BHSERVICE_H_
#define _BHSERVICE_H_

#include "config.h"
#include <pthread.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#if defined(FREEBSD) || defined(NETBSD) || defined(OPENBSD)
#include <sys/linux_ioctl.h> 
#endif 
#include <sys/ioctl.h>
#include <cstdio>
#include <cerrno>
#include <cstring>
#include <sys/socket.h>
#include <net/if.h>
#include <errno.h>

#ifndef _BHGRP_H_
#include "bhgrp.h"
#endif

#include "pos.h"

namespace BhPos{

/**
 * @defgroup bhservice Services
 * @{
 *
 * @ingroup commlib
 * @{	
 */
namespace BhPosService{

/* }@
 *
 * @ingroup bhservice
 * @{
 */
	
class ServiceException
{           
public:
   enum ErrorCode{ 
      seMemory, seBadProtocol, seServiceByName, 
      seBadHostName, seNoIPAddress, seDNSError,
      seMutexLock, seParameter, seOption,
      seNoProtocolSupport, seKernelMemory, seNoDescriptors,
      sePermissionDenied, seInvalidProtocol,
      seBadDescriptor, seBadPointer, seAlreadyConnected,
      seConnectRefused, seConnectTimeOut, seNetUnreachable,
      seAddrInUse, seInProgress, seAlreadyConnecting,
      seIncorrectAddrFamily, seBrdCastNotEnabled, seAlreadyBound,
      seAddressProtected, seCantListen, seNotStreamSock,
      seNoPendingConnections, seFirewall, seNotConnected,
      seWouldBlock, seIntseupted, seMessageSizeTooBig, 
      seNotBound, seOptionNotSupported,
      seWhoCares, seTotalWanker, seUnknown };

   ServiceException(ErrorCode type);
   ServiceException(const ServiceException&);
   ~ServiceException();

   operator const ErrorCode() { return type; }

private:
   ErrorCode  type;
   char       *ErrorString;

   void SetString(const char *pcErrString);
};

/* }@
 *
 * @ingroup bhservice
 * @{
 */

class IPAddress
{
private:
   int  type;
   int  len;
   char *name;
   char address[16];
   char *address_string;

   static pthread_mutex_t  ipmutex;

   void setname(const char *n);
   void setaddress(const char *a, const int l);
   
public:
   IPAddress();
   ~IPAddress();

   IPAddress& operator=(const IPAddress& ipa);
   IPAddress& operator=(const char *name);
   IPAddress& operator=(const in_addr i4a);
   IPAddress& operator=(const in6_addr i6a);

   bool operator==(const IPAddress &ipa) const;
   bool operator==(const char *name) const;
   bool operator==(const in_addr i4a) const;
   bool operator==(const in6_addr i6a) const;

   operator const char*() { return name; }

   void SetName(const char *, bool bIPv6);
   void SetAddress(const char *, bool bIPv6);

   const int   GetAddressFamily()  { return type; }
   const char *GetName()           { return name; }
   const void *GetAddress()        { return (void *) address; }
   const int   GetAddressLen()     { return len; }
   const char *GetAddressString()  { return address_string; }
};
/* }@
 *
 * @ingroup bhservice
 * @{
 */

typedef int (*BHNET_CALLBACK)(int, int, int, void *);

/* }@
 *
 * @ingroup bhservice
 * @{
 */

enum BhServiceRequestStatus{
   BHSERVERR_NONE,
	BHSERVERR_INPROGRESS,
   BHSERVERR_TIMEDOUT,
   BHSERVERR_REFUSED,
	BHSERVERR_GRANTED
};

/* }@
 *
 * @ingroup bhservice
 * @{
 */

enum BhserviceOption{
	BHSERVICE_OPT_RESEND,
	BHSERVICE_OPT_RETRIES,
	BHSERVICE_OPT_TIMEOUT,
	BHSERVICE_OPT_BACKOFF
};

/* }@
 *
 * @ingroup bhservice
 * @{
 */

class CNetwork{

private:
	bool          bTcp;
	ip_mreq       mreq;
	sockaddr_in   sin;
	sockaddr_in   posserv;
	unsigned char macaddr[6];
	
	bool  bHaveServerAddress;
	
	bool  bCallback;
	bool  bBackoff;
	float backoff_value;
	int   retries;
	int   max_retries;
	bool  bResend;
	bool  bTimeout;
	int   timer_value;
	
	socklen_t  size;
	timeval	 to;

protected:
	BhPosHdr   	txhdr;
	BhPosHdr   	rxhdr;
	BhServiceRequestStatus status;
	int				err;
   int            handle;
   int            Receive();
   void           TimerValue(timeval *);
   BHNET_CALLBACK cb;

public:
 enum ConnectType{
      BHNET_UDP,
      BHNET_TCP,
      BHNET_MC
   };

	int	sd;

	CNetwork(int, IPAddress&, BHNET_CALLBACK);
	~CNetwork();
	int  Open();
	void Close();
	int  GetMac(unsigned char *);
	int  Send(BhPosHdr *);
	int  ReSend();

	int GetOpt(BhserviceOption, void *);
	int SetOpt(BhserviceOption, void *);
	
	int  SetServer(int, char *, unsigned short);
	int  SetServer(IPAddress&);
	int  SetCtrl(int, char *, unsigned short);
	
	int Timeout();
//	void Activity(int d, GdkInputCondition c);
//	void Ctrl_Activity(int d, GdkInputCondition c);
	int  Send(BhPosHdr *hdr, bool bRs, bool bTo, int to_val);
};

/* }@
 *
 * @ingroup bhservice
 * @{
 */

class CServiceRequest : public CNetwork
{
private:
	BhPosHdr msg;

protected:
	void PrepHdr();

public:
   CServiceRequest(int, IPAddress&, BHNET_CALLBACK);
   ~CServiceRequest();
	int GetOption(BhserviceOption, void *);
	int SetOption(BhserviceOption, void *);
   int GetError();
	BhServiceRequestStatus GetStatus();

   int GetSite(CSite&);
	int Ping();
	int Shutdown(int, int);
};

/* }@
 *
 * @ingroup bhservice
 * @{
 */

class CClientAck{
private:
   int   client;
   bool  bAck;
   
public:
   CClientAck();
   ~CClientAck();
   CClientAck(int, bool);
   CClientAck(const CClientAck& ca);
   CClientAck &operator=(const CClientAck& ca);
   bool operator==(const CClientAck& ca);
   bool operator!=(const CClientAck& ca);
   bool operator==(const int i);
   bool operator!=(const int i);
   bool operator<(const CClientAck& ca);
   bool Status();
   void Ack();
   void Clear();
};

/* }@
 *
 * @ingroup bhservice
 * @{
 */

class CClientAckCollector
{
private:
   std::list<CClientAck> clients;
   int current;
   int site;
   int dept;
   int group;
   int post;

public:
   CClientAckCollector();
   CClientAckCollector(int s, int d, int g, int p);
   ~CClientAckCollector();
   void Clear();
   void Reset();
   int Add(CClientAck& ca);
   int Ack(int);
   int Status();
   int GetNext();
   int NumLeft();
   int GetSite() { return site; }
   int GetDepartment() { return dept; }
   int GetGroup() { return group; }
   int GetPost() { return post; }
};
/* }@ */
}
/* }@ 
	}@ */
}

#endif //   _BHSERVICE_H_
