/*
ckeymap.h
Copyright (C)2002  Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 

   Part of:
   BhPos, Copyright (C) 2001 Al Kingsley
   BhPos comes with ABSOLUTELY NO WARRANTY.

*/
#ifndef _CKEYMAP_H_
#define _CKEYMAP_H_
#include <ctime>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>

#include <libxml/tree.h>
#include <libxml/parser.h>

#include <list>
#include <algorithm>

#include "bhkevent.h"

namespace BhPos{

/**
 * @defgroup kmap Keymaps
 * @{
 *
 * @ingroup commlib
 * @{	
 */

/////////////////////////////////////

enum BhKeyTargetType{
   BHKMAP_TARGET_TYPE_NONE=0,
   BHKMAP_TARGET_TYPE_KEY,
   BHKMAP_TARGET_TYPE_PLU,
   BHKMAP_TARGET_TYPE_FUNCTION
};

/* }@
 *
 * @ingroup kmap
 * @{
 */

#define BHKMAP_TARGET_AREA     0x00010000
#define BHKMAP_TARGET_IMAGE    0x00020000

/* }@
 *
 * @ingroup kmap
 * @{
 */

class CMappedKeyRect{
protected:
   int x, y, xsize, ysize;
   
public:
   CMappedKeyRect();
   CMappedKeyRect(int, int, int, int);
   ~CMappedKeyRect();
   bool operator==(CMappedKeyRect&);
   void operator+=(CMappedKeyRect&);
   void operator-=(CMappedKeyRect&);
   int  Set(int, int, int, int);
   int  StartX() const;
   int  StartY() const;
   int  Width() const;
   int  Height() const;
   bool Inside(int, int);
   int  Add(CMappedKeyRect&);
   int  Subtract(CMappedKeyRect&);
};

/* }@
 *
 * @ingroup kmap
 * @{
 */

class CMappedKeyImage{
protected:

public:
   CMappedKeyImage();
   ~CMappedKeyImage();
   int Set(char *, char *, char *);
   
};

/* }@
 *
 * @ingroup kmap
 * @{
 */

enum BhKeyMapped{
   CMAPPEDKEY_TYPE_PUSHBUTTON,
   CMAPPEDKEY_TYPE_TOGGLED,
   CMAPPEDKEY_TYPE_RADIO,
   CMAPPEDKEY_TYPE_TIMED,
   CMAPPEDKEY_TYPE_PERIODIC
};

/* }@
 *
 * @ingroup kmap
 * @{
 */

enum BhMappedKeyState{
   CMAPPEDKEY_STATE_UNKNOWN,
   CMAPPEDKEY_STATE_OFF,
   CMAPPEDKEY_STATE_OVER,
   CMAPPEDKEY_STATE_ON,
   CMAPPEDKEY_STATE_RADIOSEL,
   CMAPPEDKEY_STATE_RADIOUNSEL,
   CMAPPEDKEY_STATE_FLASHING,
   CMAPPEDKEY_STATE_ENABLED=0x00001000
};

/* }@
 *
 * @ingroup kmap
 * @{
 */

#define CMAPPEDKEY_STATE_ENABLED    0x00001000

/* }@
 *
 * @ingroup kmap
 * @{
 */

enum BhKeymapParam{
   KM_PARAM_TYPE,
   KM_PARAM_SOURCE,
   KM_PARAM_SOURCEMOD,
   KM_PARAM_TARGETLEN,
   KM_PARAM_TARGET,
   KM_PARAM_TARGETFUNC,
   KM_PARAM_TARGETMOD,
   KM_PARAM_TARGETTYPE,
   KM_PARAM_GROUP,
   KM_PARAM_RECT,
   KM_PARAM_X,
   KM_PARAM_Y,
   KM_PARAM_XSIZE,
   KM_PARAM_YSIZE,
   KM_PARAM_IMAGE,
   KM_PARAM_IMGOFF,
   KM_PARAM_IMGOVER,
   KM_PARAM_IMGON,
   KM_PARAM_IMGMANIP
};

/* }@
 *
 * @ingroup kmap
 * @{
 */

class CMappedKey{
protected:
   int             key_type;
   unsigned int    key;
   unsigned int    src_modifiers;
   BhKeyTargetType type;
   int             target_len;
   unsigned int   *target;
   BhKeyEvent      target_func;
   unsigned int    trg_modifiers;
   
   int  group;
   int  state;
   bool bImage;
   bool bRect;
   CMappedKeyImage *image;
   CMappedKeyRect  *rect;
   
public:
   CMappedKey();
   CMappedKey(CMappedKey const&);
   CMappedKey(BhKeyTargetType type, unsigned int k, unsigned int m, unsigned int, unsigned int *t, int l);
   virtual ~CMappedKey();
   void Close();
   CMappedKey &operator=(CMappedKey const&);
   bool operator==(CMappedKey const&);
   bool operator!=(CMappedKey const&);
   int  SetParameter(BhKeymapParam, void *);
   int  GetParameter(BhKeymapParam, void *);
   void Set(BhKeyTargetType, unsigned int, unsigned int, unsigned int, unsigned int *, int);
   int  SetArea(int, int, int, int);
   int  SetArea(CMappedKeyRect&);
   int  Image(CMappedKeyImage&);
   unsigned int *Get(unsigned int, unsigned int src_mod, unsigned int *trg_mod);
   unsigned int Key(){ return key; }
   BhKeyTargetType Type(){ return type; }
   int  TargetLen(){ return target_len; }
   unsigned int TargetKey();
   unsigned int TargetModifier();
   int  GetPrintableTarget(wchar_t *);
};

/* }@
 *
 * @ingroup kmap
 * @{
 */

////////////////////////////////////
//
//   struct GdkEventKey
//   {
//     GdkEventType type;
//     GdkWindow *window;
//     gint8 send_event;
//     guint32 time;
//     guint state;
//     guint keyval;
//     gint length;
//     gchar *string;
//     guint16 hardware_keycode;
//     guint8 group;
//   };
//
//   GDK_SHIFT_MASK   the Shift key. 
//   GDK_LOCK_MASK    a Lock key (this may either be CapsLock or ShiftLock). 
//   GDK_CONTROL_MASK the Control key. 
//   GDK_MOD1_MASK    the fourth modifier key (normally the Alt key). 
//   

/* }@
 *
 * @ingroup kmap
 * @{
 */
 
class CKeyMap{
private:
   std::list<CMappedKey> map;

protected:

public:
   CKeyMap();
   virtual ~CKeyMap();
   //int Load(wchar_t *);
   //int Save(wchar_t *);
   int NumKeys();
   BhKeyTargetType IsMapped(unsigned int key, unsigned int modifier, CMappedKey& k);
   BhKeyTargetType IsMapped(GdkEventKey *, CMappedKey& k);
   int Get(int, CMappedKey&);
   int Get(CMappedKey&);
   int Add(CMappedKey&);
   int Edit(CMappedKey&, CMappedKey&);
   int Delete(CMappedKey&);
};

/* }@
 *
 * @ingroup kmap
 * @{
 */

namespace Parser{

/* }@
 *
 * @ingroup kmap
 * @{
 */
	
#define KMP_PARAM_NAME             1
#define KMP_PARAM_MANU             2
#define KMP_PARAM_MODEL            3
#define KMP_PARAM_IF               4
#define KMP_PARAM_LICENCE          5
#define KMP_PARAM_COPYRIGHT        6
#define KMP_PARAM_CREATEWHO        7
#define KMP_PARAM_CREATEWHEN       8
#define KMP_PARAM_CREATEWHENSTRING 9
#define KMP_PARAM_MODWHO           10
#define KMP_PARAM_MODWHEN          11
#define KMP_PARAM_MODWHENSTRING    12
#define KMP_PARAM_PROGKEYS         13
#define KMP_PARAM_TS               14 
#define KMP_PARAM_TSX              15
#define KMP_PARAM_TSY              16
#define KMP_PARAM_TSXRES           17
#define KMP_PARAM_TSYRES           18

/* }@
 *
 * @ingroup kmap
 * @{
 */
 
class CKeymapParser
{
private:
   char     *filename;
   bool      bOpen;
   xmlNsPtr  ns;
   xmlDocPtr doc;
   
   unsigned char *convert(char *, char *);
   
protected:
   char *name;
   char *manufacturer;
   char *model;
   char *interface;
   char *licence;
   char *copyright;
   char *createwho;
   char *modwho;
   
   tm	 createwhen;
   tm	 modwhen;
   
   int   num_prog_keys;
   
   bool bTouchscreen;
   int  disp_type;
   int  disp_x;
   int  disp_y;
   int  res_x;
   int  res_y;
   
   void ParseKeymap(xmlNodePtr node, CKeyMap&);
   void ParseKey(xmlNodePtr node, CKeyMap&);
   void ParseTouchScreen(xmlNodePtr node, CMappedKey& map);
   unsigned int CharToKey(char *c);
   char KeyToChar(unsigned int);
   unsigned int ModiferValue(xmlChar *val);
   int ValueModifier(unsigned int, xmlChar *val);
   xmlNodePtr FindNode(int, CMappedKey& k);
   
public:
   CKeymapParser();
   ~CKeymapParser();
   int  New();
   int  Open(const char *fname, CKeyMap&);
   void Close();
   int  Save(const char *fname, CKeyMap&);
   int  SetParameter(unsigned int, void *);
   int  GetParameter(unsigned int, void *);
   int  AddKeyNode(CMappedKey& k);
   int  DeleteKeyNode(CMappedKey& k);
   int  ModifyKeyNode(CMappedKey& k);
   int  ModifyProps(unsigned int, void *);
   void Validate(char *);
};

/* }@  */

}

/* }@ 
	}@ */
}

#endif // _CKEYMAP_H_
